/**************************************************
 *                                                *
 *  JBoss Labs: Creating Professional Open Source *
 *                                                *
 *   Distributable under LGPL license.            *
 *   See terms of license at gnu.org.             *
 *                                                *
 *************************************************/
package org.jboss.wiki;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

/**
 * <p>
 * </p>
 * 
 */
public class FileDataSource implements MediaDataSource {

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */
	private final String SAVE_COMMENT = "Saved by FileDataSource of JBoss Wiki";

	private final String propFileName = "filedatasource.properties";// "/home/tomek/java/sandbox/new_svn/forge/portal-extensions/forge-wiki/src/java/org/jboss/wiki/filedatasource.properties";

	private String pathToMedia;

	private String pathToAttachments;

	private Properties fileDSProps;

	private final int BUF_SIZE = 32768;

	public FileDataSource() {
		System.out.println("Looking for props file");

		fileDSProps = new Properties();

		try {
			fileDSProps.load(FileDataSource.class
					.getResourceAsStream(propFileName));
		} catch (IOException ioe) {
			System.err.println("Can't load the file " + propFileName + "\n"
					+ ioe);
		}

		loadProperties();

		File pathToMDir = new File(pathToMedia);
		
		if (!pathToMDir.exists()) {
			//creating file structure to wiki
			System.out.println("Creating file structure !");
			
			if (!pathToMDir.mkdirs()) {
				System.err.println("Couldn't create file structure. Check the prmissions !!!");
			}
		}
	}

	public void loadProperties() {
		if (fileDSProps.getProperty("pathToMedia") != null) {
			pathToMedia = fileDSProps.getProperty("pathToMedia");
			System.out.println("Path to media is: " + pathToMedia);
			pathToAttachments = fileDSProps.getProperty("pathToAttachements");
			System.out.println("Path to attachements is: " + pathToAttachments);
		} else {
			System.err
					.println("Your properties file lacks of pathToMedia property. It will not run properly ! Please update "
							+ propFileName);
		}
	}

	public boolean preSave() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean savePage(WikiPage page, String languageCode) {
		// TODO Languages

		boolean status = false;

		if (pageExists(page.getName())) { // page exists on harddisk - add new
			// version
			Properties pageProps = getPageProps(page.getName());

			pageProps.setProperty(page.getLastVersion() + ".author", page
					.getLastAuthor().getName());

			try {
				pageProps.store(new FileOutputStream(new File(pathToMedia
						+ "/OLD/" + page.getName() + "/page.properties")),
						"Saved by FileDataSource of JBoss Wiki");
			} catch (Exception e) {
				System.err
						.println("[FILEDATASOURCE]: Cannot save property file: "
								+ e.toString());
			}

			System.out.println("Opening files");
			File pageFile = new File(pathToMedia + "/OLD/" + page.getName()
					+ "/" + (page.getLastVersion() - 1) + ".txt");
			File pageOldFile = new File(pathToMedia + "/" + page.getName()
					+ ".txt");

			FileOutputStream pageFileWriter;
			FileInputStream pageFileReader;

			try {
				pageFileWriter = new FileOutputStream(pageFile);
				pageFileReader = new FileInputStream(pageOldFile);

				int b;

				while ((b = pageFileReader.read()) != -1) {
					pageFileWriter.write(b);
				}

				pageFile.setLastModified(pageOldFile.lastModified());

				pageFileWriter.close();

				pageOldFile.delete();

				pageFileWriter = new FileOutputStream(pageOldFile);

				String pageContent = page.getContent();

				for (int i = 0; i < pageContent.length(); i++) {
					pageFileWriter.write((int) pageContent.charAt(i));
				}

				pageFileWriter.close();
				status = true;
			} catch (FileNotFoundException fnfe) {
				System.err.println("[FILEDATASOURCE]: Cannot save page: "
						+ fnfe.toString());
			} catch (IOException ioe) {
				System.err.println("[FILEDATASOURCE]: Cannot save page: "
						+ ioe.toString());
			}

		} else { // there is no page on disk. We have to create new one.

			try {

				// create needed dirs
				File newPageFile = new File(pathToMedia + "/OLD/"
						+ page.getName());
				newPageFile.mkdirs();

				// write the page
				newPageFile = new File(pathToMedia + "/OLD/" + page.getName()
						+ "/page.properties");

				FileOutputStream pageWriter = new FileOutputStream(newPageFile);

				Properties properties = new Properties();
				properties.setProperty(page.getLastVersion() + ".author", page
						.getLastAuthor().getName());

				properties.store(pageWriter, SAVE_COMMENT);

				pageWriter.close();

				pageWriter = new FileOutputStream(new File(pathToMedia + "/"
						+ page.getName() + ".txt"));

				String pageContent = page.getContent();

				for (int i = 0; i < pageContent.length(); i++) {
					pageWriter.write((int) pageContent.charAt(i));
				}

				pageWriter.close();

				status = true;
			} catch (Exception e) {
				System.err.println("Cannot write new page: " + e);
				status = false;
			}
		}
		return status;
	}

	public boolean postSave() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean preGet() {
		// TODO Auto-generated method stub
		return false;
	}

	public Properties getPageProps(String pageName) {
		Properties pageProps = new Properties();

		try {
			FileInputStream fpi = new FileInputStream(new File(pathToMedia
					+ "/OLD/" + pageName + "/page.properties"));
			pageProps.load(fpi);
			fpi.close();
		} catch (IOException ioe) {
			// There is no prop file for some reson - create one.

			File pagePropsFile = new File(pathToMedia + "/OLD/" + pageName);

			// create dirs
			pagePropsFile.mkdirs();

			// create file
			pagePropsFile = new File(pathToMedia + "/OLD/" + pageName
					+ "/page.properties");

			try {
				pagePropsFile.createNewFile();

				if (pageExists(pageName)) {
					// if page exists it looks like there is lack of the props structure
					FileOutputStream fpo = new FileOutputStream(pagePropsFile);

					pageProps.setProperty("1.author", "unknown");
					pageProps.store(fpo, SAVE_COMMENT);
					fpo.close();
				}

			} catch (IOException e) {
				System.err.println("Cannot create page props file for page: "
						+ pageName);
			}
			return pageProps;
		}

		return pageProps;
	}

	public Properties getAttProps(String pageName, String attName) {
		Properties attProps = new Properties();

		File propFile = new File(pathToAttachments + "/" + pageName + "-att/"
				+ attName + "-dir/");

		if (!propFile.exists()) {
			// create dirs for this property file
			propFile.mkdirs();
		}

		propFile = new File(pathToAttachments + "/" + pageName + "-att/"
				+ attName + "-dir/attachment.properties");

		try {
			attProps.load(new FileInputStream(propFile));
		} catch (IOException ioe) {
			System.err
					.println("Cannot read attachment.properties for attachement: "
							+ pathToAttachments
							+ "/"
							+ pageName
							+ "-att/"
							+ attName + "-dir/attachment.properties\n" + ioe);
		}

		return attProps;
	}

	public void saveAttProps(Properties props, String pageName, String attName) {
		try {
			props.store(new FileOutputStream(new File(pathToAttachments + "/"
					+ pageName + "-att/" + attName
					+ "-dir/attachment.properties")), SAVE_COMMENT);
		} catch (Exception ioe) {
			System.err
					.println("Cannot store attchement.properties for attachement: "
							+ pathToAttachments
							+ "/"
							+ pageName
							+ "-att/"
							+ attName + "-dir/attachments.properties\n" + ioe);
		}

	}

	public WikiPage getPage(String pageName) {
		// System.out.println("[WIKI:FileDataSource]: Getting new page: " +
		// pageName);

		WikiPage page = null;

		File pageFile = new File(pathToMedia + "/" + pageName + ".txt");

		if (pageFile.exists()) {

			Properties pageProps = getPageProps(pageName);

			if (pageProps != null) {

				int i = 1;

				// get the author of the latest version of page
				while (pageProps.getProperty(String.valueOf(i + 1) + ".author") != null) {
					i++;
				}

				Credentials author = new Credentials(pageProps
						.getProperty(String.valueOf(i) + ".author"));

				StringBuilder pageContent = new StringBuilder();

				try {
					FileInputStream input = new FileInputStream(pageFile);

					int nextChr = input.read();

					while (nextChr != -1) {
						pageContent.append((char) nextChr);
						nextChr = input.read();
					}
				} catch (IOException ioe) {
					System.err.println(ioe);
				}

				page = new WikiPage(pageName, author, pageContent.toString(),
						i, i, new Date(pageFile.lastModified()), this);

				page.setLength(pageFile.length());
			}
		}

		return page;

	}

	public boolean postGet() {
		// TODO Auto-generated method stub
		return false;
	}

	public WikiPage getPage(String pageName, String languageCode) {
		if (languageCode != null) {
			System.out
					.println("As for now, languages aren't supported in the wiki. Please do not use language codes");
		}
		return getPage(pageName);
	}

	public WikiPage getPageAtVersion(WikiPage originPage, boolean loadContent,
			String languageCode, int version) {
		if (languageCode != null) {
			System.out
					.println("As for now, languages aren't supported in the wiki. Please do not use language codes");
		}

		return getPageAtVersion(originPage, loadContent, version);
	}

	public WikiPage getPageAtVersion(WikiPage originPage, boolean loadContent,
			int version) {

		/*
		 * Calendar cal = Calendar.getInstance(); cal.setTime(new Date()); long
		 * oldMilis = cal.getTimeInMillis();
		 */

		WikiPage page = null;

		try {
			page = (WikiPage) originPage.clone();
		} catch (CloneNotSupportedException e) {
			System.err.println(e);
		}

		if (page != null) {
			if (page.getLastVersion() < version) {
				System.out.println("There is no version " + version
						+ " of page " + page.getName());
				return null;
			}

			getContentAtVersion(page, loadContent, version);

			page.setLastAuthor(getAuthorAtVersion(page.getName(), version));

			page.setVersion(version);

			page.setEditDate(getDateAtVersion(page.getName(), version));

		}

		/*
		 * cal.setTime(new Date());
		 * 
		 * System.out.println("PAGE "+page.getName()+" loaded in:
		 * "+(cal.getTimeInMillis() - oldMilis));
		 */
		return page;
	}

	public void getContentAtVersion(WikiPage page, boolean loadContent,
			int version) {
		StringBuilder ret = null;

		File pageFile = new File(pathToMedia + "/OLD/" + page.getName() + "/"
				+ version + ".txt");

		if (loadContent) {
			try {
				FileInputStream input = new FileInputStream(pageFile);

				int nextChr = input.read();

				ret = new StringBuilder();

				while (nextChr != -1) {
					ret.append((char) nextChr);
					nextChr = input.read();
				}

				page.setPageContent(ret.toString());
				page.setLength(ret.length());

			} catch (IOException ioe) {
				System.err.println(ioe);
			}
		} else {
			page.setLength(pageFile.length());
			page.setPageContent(null);
		}

	}

	public Date getDateAtVersion(String pageName, int version) {
		Date ret = null;

		File pageFile = new File(pathToMedia + "/OLD/" + pageName + "/"
				+ version + ".txt");

		ret = new Date(pageFile.lastModified());

		return ret;
	}

	public Credentials getAuthorAtVersion(String pageName, int version) {
		return new Credentials(getPageProps(pageName).getProperty(
				version + ".author"));
	}

	public boolean pageExists(String pageName) {
		File pageFile = new File(pathToMedia + "/" + pageName + ".txt");

		if (pageFile.exists()) {
			return true;
		} else {
			return false;
		}
	}

	public void addAtachement(File attFile, String attName, WikiPage page,
			String user) {
		String pageName = page.getName();
		Properties attProps = getAttProps(pageName, attName);

		int lastVersion = 1;

		if (attProps.size() > 0) {
			// find last version of this file
			while (attProps.getProperty(lastVersion + ".author") != null) {
				lastVersion++;
			}
		}

		String[] extension = attName.split("\\.");
		String fileName = pathToAttachments
				+ "/"
				+ pageName
				+ "-att/"
				+ attName
				+ "-dir/"
				+ lastVersion
				+ "."
				+ ((attName.indexOf('.') != -1) ? extension[extension.length - 1]
						: "bin");

		FileInputStream oldFile = null;
		FileOutputStream newFile = null;

		try {
			newFile = new FileOutputStream(new File(fileName));
			oldFile = new FileInputStream(attFile);
		} catch (FileNotFoundException e) {
			System.err.println(e);
		}

		// writing old file to new one
		try {
			byte[] buffer = new byte[BUF_SIZE];
			int read;
			while ((read = oldFile.read(buffer)) != -1)
				newFile.write(buffer, 0, read);
		} catch (Exception e2) {
			// Nothing that we can really do. Just send an incomplete
			// file.
		} finally {
			try {
				newFile.close();
				oldFile.close();
			} catch (IOException e) {
				System.err.println(e);
			}
		}

		// at the end delete the temp file
		attFile.delete();

		attProps.setProperty(lastVersion + ".author", user);

		saveAttProps(attProps, pageName, attName);
	}

	public Set<String> getAttachementsSet(WikiPage page) {
		Set<String> attSet = new HashSet<String>();

		File attDir = new File(pathToAttachments + "/" + page.getName()
				+ "-att");

		if (attDir.exists()) {
			File[] attachements = attDir.listFiles();

			for (int i = 0; i < attachements.length; i++) {
				if ((attachements[i].isDirectory())
						&& (attachements[i].getName().endsWith("-dir"))) {
					String[] tokens = attachements[i].getName().split("/");
					attSet.add(tokens[tokens.length - 1].substring(0,
							tokens[tokens.length - 1].length() - 4));
				}
			}
		} else {
			// page has no attachements
			return null;
		}

		return attSet;
	}

	public Set<String> getAllPageNames() {
		Set<String> attSet = new HashSet<String>();

		File attDir = new File(pathToMedia + "/");

		File[] pageFiles = attDir.listFiles();

		for (int i = 0; i < pageFiles.length; i++) {
			if ((pageFiles[i].isFile())
					&& (pageFiles[i].getName().endsWith(".txt"))) {
				String[] tokens = pageFiles[i].getName().split("/");
				attSet.add(tokens[tokens.length - 1].substring(0,
						tokens[tokens.length - 1].length() - 4));
			}
		}

		return attSet;
	}

}
