/**************************************************
 *                                                *
 *  JBoss Labs: Creating Professional Open Source *
 *                                                *
 *   Distributable under LGPL license.            *
 *   See terms of license at gnu.org.             *
 *                                                *
 *************************************************/
package org.jboss.wiki;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.portlet.*;
import org.jboss.portlet.*;
import org.jboss.wiki.fileaccess.WikiFileAccessFilter;
import org.apache.commons.fileupload.PortletDiskFileUpload;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.FileItem;

/**
 * The WikiPortlet.
 * <p>
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szimanski@jboss.com">Tomasz Szimanski</a>
 * @version $Id$
 */

public class WikiPortlet extends JBossPortlet {

	final static String jspPath = "/WEB-INF/jsp/";

	private String defaultPage = "Main";

	private WikiEngine wikiEngine;

	public void init() {
		wikiEngine = new WikiEngine();
		
		//WikiCommon wikiCommon = new WikiCommon();
		
		//WikiServiceMenagement wikiServiceMenagement = wikiCommon.getWikiManagement();
		
		//wikiServiceMenagement.setWikiEngine(wikiEngine);
	}

	public void processAction(JBossActionRequest rReq, JBossActionResponse rResp) {
		
		// wikiContext is null - to get refernece to actual page

		WikiContext wikiContext = null;

		if (PortletDiskFileUpload.isMultipartContent(rReq)) {
			PortletDiskFileUpload dfu = new PortletDiskFileUpload();

			// maximum allowed file upload size (10 MB)
			dfu.setSizeMax(10 * 1000 * 1000);

			// maximum size in memory (vs disk) (100 KB)
			dfu.setSizeThreshold(100 * 1000);

			try {
				// get the FileItems
				List fileItems = dfu.parseRequest(rReq);
				Iterator iter = fileItems.iterator();
				while (iter.hasNext()) {
					FileItem item = (FileItem) iter.next();
					if (item.isFormField()) {
						// pass along to render request
						String fieldName = item.getFieldName();
						String value = item.getString();
						rResp.setRenderParameter(fieldName, value);
					} else {
						// write the uploaded file to a new location
						String fieldName = item.getFieldName();
						String fileName = item.getName();
						// String contentType = item.getContentType();
						// long size = item.getSize();
						/*
						 * rResp .setRenderParameter("size",
						 * Long.toString(size));
						 * rResp.setRenderParameter("contentType", contentType);
						 */
						String tempDir = System.getProperty("java.io.tmpdir");
						String serverFileName = fieldName + "-portlet.tmp";

						File serverFile = new File(tempDir, serverFileName);

						item.write(serverFile);
						/*
						 * rResp.setRenderParameter("serverFileName",
						 * serverFileName);
						 */
						getPortletContext().log(
								"serverFileName : " + tempDir + "/"
										+ serverFileName + " filename: "
										+ fileName);

						WikiPage pageWithAtt = wikiEngine.getByName(rReq
								.getParameter("page"), null);
						pageWithAtt.addAttachement(serverFile, fileName, rReq
								.getUser().getUserName());
					}
				}
			} catch (FileUploadException fue) {
				String msg = "File Upload Exception: " + fue.getMessage();
				rResp.setRenderParameter("error", msg);
				getPortletContext().log(msg, fue);
			} catch (Exception e) {
				String msg = "Exception: " + e.getMessage();
				rResp.setRenderParameter("error", msg);
				getPortletContext().log(msg, e);
			}
		}

		if (rReq.getParameter("page") != null) {
			rResp.setRenderParameter("page", rReq.getParameter("page"));

		}

		if (rReq.getParameter("searchQuery") != null) {
			// TODO do it man !
			Set<String> pages = wikiEngine.findPages(rReq
					.getParameter("searchQuery"));

			rReq.getPortletSession().setAttribute("findResult", pages);

			// show the results
			rResp.setRenderParameter("page", "FindResult");
		}

		if (rReq.getParameter("action") != null) {
			rResp.setRenderParameter("action", rReq.getParameter("action"));
		}

		if (rReq.getParameter("type") != null) {
			rResp.setRenderParameter("type", rReq.getParameter("type"));
		}

		if (rReq.getParameter("version") != null) {
			System.out.println("[process action]Getting page "
					+ rReq.getParameter("page") + " at version "
					+ rReq.getParameter("version"));
			rResp.setRenderParameter("version", rReq.getParameter("version"));
		}

		if (rReq.getParameter("editAction") != null) {
			System.out.println("EDITING PAGE");

			// unlock page
			WikiPage edPage = wikiEngine.getByName((String) rReq
					.getParameter("editedPage"), wikiContext);

			if (edPage != null) { // it may be null if the page isn't saved
				// yet
				edPage.unlock(rReq.getUser());
			}

			// saving, previewing or canceling

			if (rReq.getParameter("editAction").equals("Save")) {
				Credentials credentials = new Credentials(
						(rReq.getUser() != null) ? rReq.getUser().getUserName()
								: "Unknown");
				System.out.println("SAVING PAGE");

				// save the page

				if (edPage == null) {
					edPage = new WikiPage(rReq.getParameter("editedPage"),
							credentials, "", 0, 0, new Date(), wikiEngine
									.getMediaDataSource());
				}

				// set new content
				edPage.setPageContent(rReq.getParameter("wikiContent"));

				// set author name for new version
				edPage.setLastAuthor(credentials);

				// increase page version
				edPage.setLastVersion(edPage.getLastVersion() + 1);

				// set new data
				edPage.setEditDate(new Date());

				// save the page
				edPage.save();

				// and finally remove old page object from memory to force
				// reloading it
				wikiEngine.refreshPage(edPage.getName());
			}
		}

	}

	public void doView(JBossRenderRequest rReq, JBossRenderResponse rRes)
			throws PortletException, java.io.IOException {

		PortletURL wikiURL = rRes.createActionURL();

		rReq.setAttribute("actionURL", wikiURL.toString());

		// Create server host and put it into session for plugins
		String hostURL = ((rReq.isSecure()) ? "https" : "http") + "://"+rReq.getServerName()+((rReq.getServerPort() == 80) ? "" : ":"+ rReq.getServerPort());
		
		rReq.getPortletSession().setAttribute("hostURL", hostURL);
		rReq.getPortletSession().setAttribute("actionURL", wikiURL.toString());
		
		rReq.getPortletSession().setAttribute("isSecure",
				new Boolean(rReq.isSecure()));

		WikiPage noSuchPage = new WikiPage("", new Credentials(""),
				"There is no such page. Click on EDIT to start it.", 0, 0,
				null, wikiEngine.getMediaDataSource());

		String page = "Wiki.jsp";

		String wikiPage = defaultPage;

		WikiPage pageToShow = null;

		WikiContext wikiContext = null;

		if (rReq.getParameter("page") != null) {
			wikiPage = rReq.getParameter("page");
		}

		if ((rReq.getParameter("action") != null)
				&& (rReq.getParameter("action").equals("Attach"))) {
			rReq.setAttribute("fileAttach", "true");
		}

		if ((rReq.getParameter("action") != null)
				&& (rReq.getParameter("action").equals("Edit"))
				&& (rReq.getUser() != null)) {
			page = "Edit.jsp";

			pageToShow = wikiEngine.getByName(wikiPage, null);

			if (rReq.getParameter("version") != null) {
				// show previous version of page

				System.out.println("Getting page " + wikiPage + " at version "
						+ rReq.getParameter("version"));
				pageToShow = wikiEngine.getByName(wikiPage, null, Integer
						.valueOf(rReq.getParameter("version")));

				if (pageToShow.getLastVersion() != pageToShow.getVersion()) {
					rReq.setAttribute("showedVersion", Integer.valueOf(rReq
							.getParameter("version")));
				}
			} else {
				pageToShow = wikiEngine.getByName(wikiPage, null);
			}

			if (pageToShow == null) {
				// start editing new page
				// wikiPage = defaultPage;

				pageToShow = new WikiPage(wikiPage, new Credentials(rReq
						.getUser().getUserName()), "", 0, 0, new Date(),
						wikiEngine.getMediaDataSource());
				// wikiEngine.getByName(wikiPage, wikiContext);
			}

			Date dateLock = pageToShow.lock(rReq.getUser());

			if (dateLock != null) {
				rReq
						.setAttribute(
								"lockMsg",
								"Page is locked by "
										+ pageToShow.getLockUser()
										+ " from "
										+ dateLock
										+ ".<br> I won't stop you from saving, but you or he can erease each other changes. Lock will expire: ");
			}

		} else if ((rReq.getParameter("action") != null)
				&& (rReq.getParameter("action").equals("Info"))) {
			page = "PageInfo.jsp";

			pageToShow = wikiEngine.getByName(wikiPage, null);

			if (pageToShow == null) { // if somebody entered a link to info
				// non-existing page
				page = "Wiki.jsp";
				try {
					pageToShow = (WikiPage) noSuchPage.clone();
				} catch (CloneNotSupportedException cnse) {
					System.out.println(cnse);
				}

				pageToShow.setName(wikiPage);
			} else {
				rReq.setAttribute("pageAtTop", pageToShow);
			}

		} else {
			page = "Wiki.jsp";

			String type = "HTML";

			if (rReq.getParameter("type") != null) {
				type = rReq.getParameter("type");
				//System.out.println("Getting type: " + type);
			}

			wikiContext = new WikiContext(rReq.getUser(), wikiEngine
					.getWikiType(type), new PortletWikiSession(rReq
					.getPortletSession()));

			if (rReq.getParameter("version") != null) {
				// show previous version of page
				System.out.println("Getting page " + wikiPage + " at version "
						+ rReq.getParameter("version"));
				pageToShow = wikiEngine.getByName(wikiPage, wikiContext,
						Integer.valueOf(rReq.getParameter("version")));

				if (pageToShow.getLastVersion() != pageToShow.getVersion()) {
					rReq.setAttribute("showedVersion", Integer.valueOf(rReq
							.getParameter("version")));
				}
			} else {
				pageToShow = wikiEngine.getByName(wikiPage, wikiContext);
			}

			if (pageToShow == null) {

				try {
					pageToShow = (WikiPage) noSuchPage.clone();
				} catch (CloneNotSupportedException cnse) {
					System.out.println(cnse);
				}

				pageToShow.setName(wikiPage);
			}

		}
		
		rReq
				.setAttribute(
						"wikiName",
						"<img src=\""+hostURL+"/wiki/"+WikiFileAccessFilter.specialPage+"/JBossWikiLogoMedium.png\" width=\"100\"  border=\"0\" style=\"vertical-align: middle\"/>");

		rReq.setAttribute("wikiContent", pageToShow.getContent());
		
		rReq.setAttribute("wikiImagesUrl", hostURL+"/wiki/"+WikiFileAccessFilter.specialPage+"/");

		rReq.setAttribute("wikiPage", wikiPage);

		rReq.setAttribute("editDate", pageToShow.getEditDate());

		rReq.setAttribute("author", pageToShow.getLastAuthor());

		rReq.setAttribute("logedIn", (rReq.getUser() != null) ? "true"
				: "false");

		rReq.setAttribute("defaultPage", defaultPage);

		// FIXME make this more elegant (ie. plugin)
		// pass friendly link
		/*Properties props = new Properties();
		props.load(WikiPlugin.class
				.getResourceAsStream("wikiPlugins.properties"));
		String href = props.getProperty("htmlTranslatorURL") + "wiki/"
				+ pageToShow.getName();
		String[] replaces = props.getProperty("sslRedirectReplaces").split(",");

		if (rReq.isSecure() && (replaces.length % 2 == 0)) {
			// chage normal url to secure
			for (int i = 0; i < replaces.length; i += 2)
				href = href.replaceAll(replaces[i], replaces[i + 1]);
		}*/

		rReq.setAttribute("friendlyLink", hostURL+"/wiki/"+pageToShow.getName());

		rRes.setTitle("Wiki");
		if (javax.portlet.WindowState.NORMAL.equals(rReq.getWindowState())
				|| javax.portlet.WindowState.MAXIMIZED.equals(rReq
						.getWindowState())) {
			rRes.setContentType("text/html");
			javax.portlet.PortletRequestDispatcher pRD = this
					.getPortletContext().getRequestDispatcher(jspPath + page);
			pRD.include(rReq, rRes);
			return;
		}
		doDispatch(rReq, rRes);
	}

}