/**************************************************
 *                                                *
 *  JBoss Labs: Creating Professional Open Source *
 *                                                *
 *   Distributable under LGPL license.            *
 *   See terms of license at gnu.org.             *
 *                                                *
 *************************************************/
package org.jboss.wiki;

import java.util.Vector;

/**
 * <p>
 * </p>
 * 
 */
/**
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a>
 * @version $Id$
 */
public class WikiType {

	protected String name;
	
	protected WikiEngine wikiEngine;

	/**
	 * <p>
	 * </p>
	 * 
	 */
	protected WikiContext wikiContext;

	protected Vector<WikiPlugin> plugins = new Vector<WikiPlugin>();


	public void addPlugin(String pluginName, String pluginClass) {

		WikiPlugin plugin = null;

		try {
			plugin = (WikiPlugin) Class.forName(pluginClass).newInstance();
		} catch (Exception e) {
			System.err.println("Error while loading plugin: " + pluginName
					+ "\nclass: " + pluginClass + "\n" + e);
		}

		if (plugin != null) {
			plugins.add(plugin);
			
			plugin.setWikiEngine(wikiEngine);
			
			plugin.setName(pluginName);
			
			// run init() function
			plugin.init();

			System.out.println("[WikiType:" + name + "]Plugin added: "
					+ pluginName + " class: " + pluginClass);
		}

	}

	/**
	 * <p>
	 * Processes given WikiPage with the apropriate plugins, connected to this WikiType
	 * </p>
	 * 
	 * 
	 * @return processed WikiPage (copy - not exact reference)
	 * @param wikiPage page to process
	 * @param wikiSession current session
	 */
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {
		// process through all plugins
		
		WikiPage newPage = null;
		
		try {
			newPage = (WikiPage)wikiPage.clone();
		}
		catch (CloneNotSupportedException cnse) {
			System.out.println("cannot clone the page: " + cnse);
		}
		
		// execute the plugins
		for (int i = 0; i < plugins.size(); i++) {
			//System.out.println("Executing plugin: "+plugins.get(i).getName());
			newPage = (plugins.get(i)).process(newPage, wikiSession);
		}
		
		return newPage;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		System.out.print("Setting new WikiType name: " + name);
		this.name = name;
	}
	
	public void setWikiEngine(WikiEngine wikiEngine) {
		this.wikiEngine = wikiEngine;
	}
}
